#include "simsbasic.h"


UniException::UniException(const QString& msg)
	:_error_store(msg) {
}

QString UniException::content() const noexcept
{
	return _error_store;
}

RtEntity::RtEntity() {}

void RtEntity::resetTemplet(const QString& name) {
	this->_templet_name = name;
}

QString RtEntity::templetName() const {
	return this->_templet_name;
}

void RtEntity::resetID(uint64_t id) {
	this->_entity_id = id;
}

uint64_t RtEntity::entityID() const {
	return this->_entity_id;
}

void RtEntity::resetName(const QString& name) {
	this->_runtime_name = name;
}

QString RtEntity::name() const {
	return this->_runtime_name;
}

void RtEntity::append(std::shared_ptr<WsComponent> ins) {
	if (this->_comps_list.contains(ins->name()))
		return;

	ins->bindEntity(this->shared_from_this());
	this->_comps_list[ins->name()] = ins;
}

void RtEntity::remove(const QString& component_type) {
	this->_comps_list.remove(component_type);
}

QList<std::shared_ptr<WsComponent>> RtEntity::components() const {
	return _comps_list.values();
}

std::shared_ptr<WsEntity> RtEntity::defaultNew() const {
	auto newx = std::make_shared<RtEntity>();
	newx->_entity_id = this->_entity_id;
	newx->_templet_name = this->_templet_name;
	newx->_runtime_name = this->_runtime_name;

	for (auto c : this->_comps_list)
		newx->append(c->defaultNew());
	return newx;
}

#include <QSet>
QList<QString> RtEntity::inputTypes() const
{
	QList<QString> types;
	for (auto c : this->_comps_list)
		types.append(c->inputTypes());
	return types.toSet().toList();
}

#include <QVariant>
#include <QJsonArray>
QList<WsRespondEntry> RtEntity::getEntryWithSignature(const WsRespondSignatureType& t) const
{
	QList<WsRespondEntry> list;
	for (auto c : this->_comps_list)
		list.append(c->getEntryWithSignature(t));
	return list;
}

QList<WsRespondEntry> RtEntity::getEntryWithInType(const QString& msg_type) const
{
	QList<WsRespondEntry> list;
	for (auto c : this->_comps_list)
		list.append(c->getEntrysWithInType(msg_type));
	return list;
}
void RtEntity::recoveryFrom(const QJsonObject& obj)
{
	this->_entity_id = obj["entity_id"].toVariant().toULongLong();
	this->_runtime_name = obj["entity_name"].toString();
	this->_templet_name = obj["entity_templet"].toString();

	auto array = obj["component_array"].toArray();
	for (auto idx = 0; idx < array.size(); ++idx) {
		auto value = array.at(idx).toObject();
		auto type = value["component_type"].toString();

		auto compins = this->_comps_list[type];
		if (!compins)
			throw new UniException(type + "Ͳڣ");

		compins->recoveryFrom(value);
	}
}

void RtEntity::saveTo(QJsonObject& obj) const
{
	obj["entity_id"] = QJsonValue::fromVariant(QVariant::fromValue(this->_entity_id));
	obj["entity_name"] = this->_runtime_name;
	obj["entity_templet"] = this->_templet_name;

	QJsonArray array;
	for (auto c : this->_comps_list)
	{
		QJsonObject comp_obj;
		comp_obj["component_type"] = c->name();
		c->saveTo(comp_obj);
		array.append(comp_obj);
	}
	obj["component_array"] = array;
}

QString EntitiesManager::templetName() const
{
	return "ʵ";
}

uint64_t EntitiesManager::entityID() const
{
	return const_id;
}

QString EntitiesManager::name() const
{
	return "ʵΨһʵ";
}

void EntitiesManager::append(std::shared_ptr<WsComponent> ins) {
	_comps_list[ins->name()] = ins;
}

std::shared_ptr<WsEntity> EntitiesManager::defaultNew() const
{
	return nullptr;
}

QList<WsRespondEntry> EntitiesManager::getEntryWithSignature(const WsRespondSignatureType& t) const
{
	QList<WsRespondEntry> list;
	for (auto c : this->_comps_list)
		list.append(c->getEntryWithSignature(t));
	return list;
}

QList<WsRespondEntry> EntitiesManager::getEntryWithInType(const QString& msg_type) const
{
	QList<WsRespondEntry> list;
	for (auto c : this->_comps_list)
		list.append(c->getEntrysWithInType(msg_type));
	return list;
}

void EntitiesManager::recoveryFrom(const QJsonObject& obj)
{
	auto array = obj["component_array"].toArray();
	for (auto idx = 0; idx < array.size(); ++idx) {
		auto value = array.at(idx).toObject();
		auto type = value["component_type"].toString();

		auto compins = this->_comps_list[type];
		if (!compins)
			throw new UniException(type + "Ͳڣ");

		compins->recoveryFrom(value);
	}
}

void EntitiesManager::saveTo(QJsonObject& obj) const
{
	QJsonArray array;
	for (auto c : this->_comps_list)
	{
		QJsonObject comp_obj;
		comp_obj["component_type"] = c->name();
		c->saveTo(comp_obj);
		array.append(comp_obj);
	}
	obj["component_array"] = array;
}

void EntitiesManager::remove(const QString& component_type)
{
	this->_comps_list.remove(component_type);
}

QList<std::shared_ptr<WsComponent>> EntitiesManager::components() const
{
	return this->_comps_list.values();
}

ImmediateCore::ImmediateCore(std::shared_ptr<WsEntity> bind) :_bind_entity(bind) {}

uint64_t ImmediateCore::entityManagerID() const
{
	return EntitiesManager::const_id;
}

QList<std::shared_ptr<WsMessage>> ImmediateCore::execute(const WsRespondSignatureType& resp_signature, std::shared_ptr<const WsMessage> in)
{
	QList<std::shared_ptr<WsMessage>> rets;
	auto list = this->_bind_entity->getEntryWithSignature(resp_signature);
	for (auto func : list) {
		func(shared_from_this(), in, rets);
	}
	return rets;
}

QList<std::shared_ptr<WsMessage>> ImmediateCore::execute(std::shared_ptr<const WsMessage> in)
{
	QList<std::shared_ptr<WsMessage>> rets;
	auto list = this->_bind_entity->getEntryWithInType(in->topicString());
	for (auto func : list) {
		func(shared_from_this(), in, rets);
	}
	return rets;
}
